######################################################################
#
# SPDX-License-Identifier: GPL-2.0+
#
# Driver for Solarflare network controllers and boards
# Copyright 2019 Solarflare Communications Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, incorporated herein by reference.
#
# You can also choose to distribute this program under the terms of
# the Unmodified Binary Distribution Licence (as given in the file
# COPYING.UBDL), provided that you have satisfied its requirements.
#
######################################################################

# Add a leading v to transform the version into an Ivy revision
MODULE_REVISION := v$(VERSION)

# Set the relative path of the Ivy base directory where ivysettings.xml is
# is located
IVY_BASE_DIR := "../ivybase"

# Configure options and location of Ant wrapper script
ANT_OPTS := -Dmodule.revision=$(MODULE_REVISION)
ANT := "$(IVY_BASE_DIR)/scripts/ant" $(ANT_OPTS)

# Configure options and location of templator tool
TEMPLATOR_TOOL := src/imported/templator-tool/templator.py
TEMPLATOR_OPTS := --define VERSION $(MODULE_REVISION)
TEMPLATOR := python $(TEMPLATOR_TOOL) $(TEMPLATOR_OPTS)

NIC_LIST := SFC9020 SFL9021 SFC9120 SFC9140 SFC9220 SFC9250
NIC_MROM_FILES := $(foreach nic,$(NIC_LIST),src/bin/$(nic).mrom)
NIC_DAT_FILES := $(foreach nic,$(NIC_LIST),src/bin/$(nic).dat)
NIC_UPDATE_FILES := $(foreach nic,SFC9220 SFC9250,src/bin/$(nic).update)


#############################################################################

.PHONY: veryclean update-dependencies retrieve publish sf-bootrom gpxe-ivy

veryclean:
	$(MAKE) -C src veryclean
	$(V)$(RM) ivy.xml ivy-resolved-artifacts.xml

update-dependencies:
	$(ANT) update-dependencies

ivy.xml: update-dependencies

retrieve $(TEMPLATOR_TOOL): ivy.xml
	$(ANT) retrieve

publish:
	$(ANT) publish

# Side-car files for encode.py
src/bin/sfc_uefi.rom:
	@mkdir -p $(@D)
	cd $(@D) && ../../$(IVY_BASE_DIR)/scripts/ivy_retrieve edk2
	$(RM) $(@D)/encode.py

src/bin/%.ini : ini/%.ini.template src/config/general.h
	@mkdir -p $(@D)
	$(TEMPLATOR) ini/$*.ini.template $@

# mrom files depend on their ini files
$(foreach nic,$(NIC_LIST),$(eval src/bin/$(nic).mrom : src/bin/$(nic).mrom.ini))

src/bin/%.mrom: .FORCE
	$(MAKE) -C src bin/$(notdir $@)

# The dependency generation means that the sub-makes cannot be done in parallel.
.NOTPARALLEL: sf-bootrom sf-bootdat
sf-bootrom: retrieve $(NIC_MROM_FILES)

gpxe-ivy: veryclean sf-bootrom publish

src/bin/encode.py :
	@mkdir -p $(@D)
	cd $(@D) && ../../$(IVY_BASE_DIR)/scripts/ivy_retrieve encode-tool
	chmod a+x $@

# Medford has separate flash partitions for uefi and gpxe.
src/bin/SFC9220.dat : src/bin/SFC9220.mrom
	cd $(@D) && /usr/bin/env python ./encode.py -C $(<F) $(@F)

src/bin/SFC9250.dat : src/bin/SFC9250.mrom
	cd $(@D) && /usr/bin/env python ./encode.py -C $(<F) $(@F)

src/bin/%.dat : src/bin/%.mrom
	cd $(@D) && /usr/bin/env python ./encode.py -j sfc_uefi.rom $*.mrom $(@F)

src/bin/%.update : src/bin/%.mrom
	cd $(@D) && /usr/bin/env python ./encode.py -C -r $(<F) $(@F)

sf-bootdat: src/bin/sfc_uefi.rom src/bin/encode.py sf-bootrom $(NIC_DAT_FILES)
sf-bootdat: $(NIC_UPDATE_FILES)

.FORCE:
