/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <stddef.h>
#include <stdio.h>
#include <assert.h>
#include <gdbmach.h>

#define STACK_DUMP_SIZE	32

/** @file
 *
 * i386 debug exception handler
 *
 * See stack.S for the stack definition and size.
 */
extern uint32_t _estack;

__asmcall void debugexcept_handler ( uint32_t intname, gdbreg_t *regs ) {
	union {
		uint32_t	*ptr;
		uint32_t	val;
	} sp;
	int	i;

	printf ( "\nEXCEPTION #%c%c\n"
		 "  EIP=%08x EFLAGS=%08x\n"
		 "  EAX=%08x EBX=%08x ECX=%08x EDX=%08x\n"
                 "  ESI=%08x EDI=%08x EBP=%08x ESP=%08x\n"
                 "  CS=%04x SS=%04x DS=%04x ES=%04x FS=%04x GS=%04x\n",
		 (intname >> 16) & 0xff, (intname >> 24) & 0xff,
                 (unsigned int)regs[GDBMACH_EIP],
		 (unsigned int)regs[GDBMACH_EFLAGS],
		 (unsigned int)regs[GDBMACH_EAX],
		 (unsigned int)regs[GDBMACH_EBX],
		 (unsigned int)regs[GDBMACH_ECX],
		 (unsigned int)regs[GDBMACH_EDX],
		 (unsigned int)regs[GDBMACH_ESI],
		 (unsigned int)regs[GDBMACH_EDI],
		 (unsigned int)regs[GDBMACH_EBP],
		 (unsigned int)regs[GDBMACH_ESP],
		 (unsigned int)regs[GDBMACH_CS],
		 (unsigned int)regs[GDBMACH_SS],
		 (unsigned int)regs[GDBMACH_DS],
		 (unsigned int)regs[GDBMACH_ES],
		 (unsigned int)regs[GDBMACH_FS],
		 (unsigned int)regs[GDBMACH_GS] );

	/* Provide a small stack dump if we are on the internal gPXE stack */
	sp.val = (uint32_t) regs[GDBMACH_ESP];
	if ((sp.val < (uint32_t) &_estack) &&
	    (sp.val > ((uint32_t) &_estack) - 0x1000)) {
		printf("  STACK:\n  ");
		for (i=0; i<STACK_DUMP_SIZE; i++) {
			printf("%08x%s", sp.ptr[i], ((i+1)%8? " ":"\n  "));
		}
	}
}

