/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#ifndef EBDA_H
#define EBDA_H

#include <config/general.h>

/*
 * Optional support for using EBDA realmode allocation
 *
 */

/*
 * EBDA is a royal pain the ass, because it permits (neh, mandates), that
 * our real mode segments bounce around base memory underneath us. There's
 * absolutely no way we can place our interrupt handlers inside text16/data16
 *
 * So instead, we split each interrupt handler into two. The top half lives
 * inside the prefix (which is always available at a constant physical address).
 * This top half searches base memory for the text16/data16 segments, updates
 * the librm/libkir private variables therein, then jumps into the bottom half
 * of the handler inside the text16 segment
 */
#ifndef EBDA_ALLOCATE

#define INTERRUPT_CODE( _label, _code )			\
	".section \".text16\", \"ax\", @progbits\n\t"	\
	".code16\n\t"					\
	"\n" #_label ":\n\t"				\
	_code "\n\t"					\
	".code32\n\t"					\
	".previous\n\t"

#define INTERRUPT_SEG rm_cs

#else

#define INTERRUPT_CODE( _label, _code )					\
	/* Top half */							\
	".section \".prefix.lib\", \"awx\", @progbits\n\t"		\
	".code16\n\t"							\
	"\n" #_label ":\n\t"						\
	/* Push flags and %cs:_real_label onto the stack */		\
	"pushfw\n\t"							\
	"pushw %%cs\n\t"						\
	"pushw $_real_" #_label "\n\t"					\
	/* Search for text16. This function will adjust */		\
	/* the stack segment pushed onto the stack, pop */		\
	/* this frame, then lret into text16 */				\
	"call find_and_jump_text16\n\t"					\
	".code32\n\t"							\
	".previous\n\t"							\
									\
	/* Bottom half */						\
	TEXT16_CODE ( "\n_real_" #_label ":\n\t" _code )

#define INTERRUPT_SEG prefix_seg

#endif /* EBDA_ALLOCATE */


#endif /* EBDA_H */





