/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <stdint.h>
#include <stdlib.h>
#include <gpxe/instance.h>
#include <gpxe/device.h>
#include <gpxe/netdevice.h>
#include <arch/i386/include/registers.h>

uint32_t pci_busdevfn_from_prefix = 0xffffffff;

/* |15  bus  8|7  device  3|2  fn  0| */
uint16_t fn_from_busdevfn ( uint16_t busdevfn ) {
	return busdevfn & ( ( 1 << 3 ) - 1 );
}
uint16_t dev_from_busdevfn( uint16_t busdevfn ) {
	return ( busdevfn >> 3 ) & ( ( 1 << 5 ) - 1 );
}
uint16_t bus_from_busdevfn( uint16_t busdevfn ) {
	return busdevfn >> 8;
}

void instance_set_pci_busdevfn ( struct i386_all_regs *ix86 ) {
	pci_busdevfn_from_prefix = ix86->regs.ecx;
}

uint32_t instance_pci_busdevfn ( void ) {
	return pci_busdevfn_from_prefix;
}

struct net_device *instance_netdevice ( void ) {
	struct net_device *netdev = NULL;

	if ( pci_busdevfn_from_prefix & 0xffff0000 ) {
		/* no PFA, e.g., PXE chainloading */
		netdev = find_netdev("net0");
	} else {
		struct net_device *nd;
		for_each_netdev (nd) {
			if (nd->dev->desc.location != pci_busdevfn_from_prefix)
				continue;
			netdev = nd;
			break;
		}
	}
	return netdev;
}

__asmcall void instance_main ( struct i386_all_regs *ix86 ) {
	instance_set_pci_busdevfn ( ix86 );
	ix86->regs.ecx = main ();
}

