/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL )

/* INT 13 wrapper */

#include <bios.h>
#include <config/general.h>

	.text
	.arch i386
	.code16


/****************************************************************************
 * INT 13 handler
 ****************************************************************************
 */
#ifdef EBDA_ALLOCATE
	.section ".prefix.lib", "awx", @progbits
	.global int13_wrapper
int13_wrapper:
	pushfw
	pushw	%cs
	pushw	$_real_int13_wrapper
	call	find_and_jump_text16
	.size	int13_wrapper, . - int13_wrapper
	.section ".text16", "ax", @progbits
_real_int13_wrapper:
#else
	.section ".text16", "ax", @progbits
	.globl int13_wrapper
int13_wrapper:
#endif
	pushw	%bp
	movw	%sp, %bp
#ifndef BCV
	/* Preserve %ax and %dx for future reference */
	pushw	%ax
	pushw	%dx
#endif
	/* Clear OF, set CF, call int13() */
	orb	$0, %al
	stc
	pushl	$int13
	pushw	%cs
	call	prot_call
	/* Don't chain if OF is set */
	jo	1f
	/* Restore original flags from stack */
	pushw	6(%bp)
	popfw
	/* Chain to previous INT 13 vector */
	pushfw
	lcall	*%cs:int13_vector
	/* Overwrite flags for iret */
1:	pushfw
	popw	6(%bp)
#ifndef BCV
	/* Fix up %dl:
	 *
	 * INT 13,15 : do nothing
	 * INT 13,08 : load with number of drives
	 * all others: restore original value
	 */
	cmpb	$0x15, -1(%bp)
	je	2f
	movb	-4(%bp), %dl
	cmpb	$0x08, -1(%bp)
	jne	2f
	pushw	%ds
	pushw	$BDA_SEG
	popw	%ds
	movb	$BDA_NUM_DRIVES, %dl
	popw	%ds
#endif
	/* Return */
2:	movw	%bp, %sp
	popw	%bp
	iret
#ifdef EBDA_ALLOCATE
	.size _real_int13_wrapper, . - _real_int13_wrapper
#else
	.size int13_wrapper, . - int13_wrapper
#endif

	.section ".text16.data", "aw", @progbits
	.globl int13_vector
int13_vector:
	.long 0
	.size int13_vector, . - int13_vector

