/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
/* At entry, the processor is in 16 bit real mode and the code is being
 * executed from an address it was not linked to. Code must be pic and
 * 32 bit sensitive until things are fixed up.
 *
 * Also be very careful as the stack is at the rear end of the interrupt
 * table so using a noticeable amount of stack space is a no-no.
 */
FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL )

#include <config/general.h>
#include <gpxe/sf_startup.h>

#undef DEBUG
#undef UEFI_CHECK

#define PNP_SIGNATURE ( '$' + ( 'P' << 8 ) + ( 'n' << 16 ) + ( 'P' << 24 ) )
#define PMM_SIGNATURE ( '$' + ( 'P' << 8 ) + ( 'M' << 16 ) + ( 'M' << 24 ) )
#define PCI_SIGNATURE ( 'P' + ( 'C' << 8 ) + ( 'I' << 16 ) + ( ' ' << 24 ) )
#define STACK_MAGIC ( 'L' + ( 'R' << 8 ) + ( 'E' << 16 ) + ( 'T' << 24 ) )
#define SMBIOS_SIGNATURE ( '_' + ( 'S' << 8 ) + ( 'M' << 16 ) + ( '_' << 24 ) )	
#define PMM_ALLOCATE 0x0000
#define PMM_FIND 0x0001
#define PMM_HANDLE_BASE ( ( ( 'F' - 'A' + 1 ) << 26 ) + \
			( ( 'E' - 'A' + 1 ) << 21 ) + \
			( ( 'N' - 'A' + 1 ) << 16 ) )
#define PMM_HANDLE_BASE_IMAGE_SOURCE \
	( PMM_HANDLE_BASE | 0x00001000 )
#define PMM_HANDLE_BASE_DECOMPRESS_TO \
	( PMM_HANDLE_BASE | 0x00002000 )
#define PCI_FUNC_MASK 0x07

/* ROM banner timeout.  Based on the configurable BANNER_TIMEOUT in
 * config.h, but converted to a number of (18Hz) timer ticks, and
 * doubled to allow for BIOSes that switch video modes immediately
 * beforehand, so rendering the message almost invisible to the user.
 */
#define ROM_BANNER_TIMEOUT ( 2 * ( 18 * BANNER_TIMEOUT ) / 10 )

/* Allow payload to be excluded from ROM size
 */
#if ROMPREFIX_EXCLUDE_PAYLOAD
#define	ZINFO_TYPE_ADxB "ADHB"
#define	ZINFO_TYPE_ADxW "ADHW"
#else
#define	ZINFO_TYPE_ADxB "ADDB"
#define	ZINFO_TYPE_ADxW "ADDW"
#endif

#define PCIBIOS_READ_CONFIG_DWORD	0xb10a
#define PCI_BAR_0			0x10
#define PCI_BAR_2			0x18

	.text
	.code16
	.arch i386
	.section ".prefix", "ax", @progbits
	.globl	_rom_start
_rom_start:
	
	.org	0x00
romheader:
	.word	0xAA55			/* BIOS extension signature */
romheader_size:	.byte 0			/* Size in 512-byte blocks */
	jmp	init			/* Initialisation vector */
checksum:
	.byte	0
#ifdef INFOBLK
	.org	0x08
	.word	infoblk0		/* Consumed by combobulate.py */
	.word	infoblk1

	.equ	num_infoblks, 0x10
	.word	num_infoblks
#endif
	.org	0x16
	.word	undiheader
	.org	0x18
	.word	pciheader
	.org	0x1a
	.word	pnpheader
	.size romheader, . - romheader
	
	.section ".zinfo.fixup", "a", @progbits	/* Compressor fixups */
	.ascii	ZINFO_TYPE_ADxB
	.long	romheader_size
	.long	512
	.long	0
	.previous

build_id:
	.long	_build_id		/* Randomly-generated build ID */

pciheader:
	.ascii	"PCIR"			/* Signature */
	.word	pci_vendor_id		/* Vendor identification */ 
	.word	pci_device_id		/* Device identification */
	.word	0x0000			/* Device list pointer */
	.word	pciheader_len		/* PCI data structure length */
	.byte	0x03			/* PCI data structure revision */
	.byte	0x02, 0x00, 0x00	/* Class code */
pciheader_image_length:
	.word	0			/* Image length */
	.word	0x0001			/* Revision level */
	.byte	0x00			/* Code type */
	.byte	0x80			/* Last image indicator */
pciheader_runtime_length:
	.word	0			/* Maximum run-time image length */
	.word	0x0000			/* Configuration utility code header */
	.word	clp_entry		/* DMTF CLP entry point */
	.equ pciheader_len, . - pciheader
	.size pciheader, . - pciheader
	
	.section ".zinfo.fixup", "a", @progbits	/* Compressor fixups */
	.ascii	ZINFO_TYPE_ADxW
	.long	pciheader_image_length
	.long	512
	.long	0
	.ascii	"ADHW"
	.long	pciheader_runtime_length
	.long	512
	.long	0
	.previous

	/* PnP doesn't require any particular alignment, but IBM
	 * BIOSes will scan on 16-byte boundaries rather than using
	 * the offset stored at 0x1a
	 */
	.align	16
pnpheader:
	.ascii	"$PnP"			/* Signature */
	.byte	0x01			/* Structure revision */
	.byte	( pnpheader_len	/ 16 )	/* Length (in 16 byte increments) */
	.word	0x0000			/* Offset of next header */
	.byte	0x00			/* Reserved */
pnpheader_checksum:
	.byte	0x00			/* Checksum */
	.long	0x00000000		/* Device identifier */
	.word	mfgstr			/* Manufacturer string */
	.word	prodstr			/* Product name */
pnpheader_devtype:
	.byte	0x02			/* Device base type code */
	.byte	0x00			/* Device sub-type code */
	.byte	0x00			/* Device interface type code */
pnpheader_devind:
	.byte	0xf4			/* Device indicator */
pnpheader_bcv:
	.word	0x0000			/* Boot connection vector */
	.word	0x0000			/* Disconnect vector */
pnpheader_bev:
	.word	bev_entry		/* Boot execution vector */
	.word	0x0000			/* Reserved */
	.word	0x0000			/* Static resource information vector*/
	.equ pnpheader_len, . - pnpheader
	.size pnpheader, . - pnpheader

/* Manufacturer string */
mfgstr:
	.asciz	"http://etherboot.org"
	.size mfgstr, . - mfgstr

/* Product string
 *
 * Defaults to PRODUCT_SHORT_NAME.  If the ROM image is writable at
 * initialisation time, it will be filled in to include the PCI
 * bus:dev.fn number of the card as well.
 */
prodstr:
	.ascii	PRODUCT_SHORT_NAME
prodstr_separator:
	.byte	0
#ifdef BCV
prodstr_mac_addr:
	.ascii	"000000000000 "
#endif
	.ascii	"(PCI "
prodstr_pci_id:
	.asciz	"xx:xx.x)"		/* Filled in by init code */
	.size prodstr, . - prodstr

undiheader:
	.ascii	"UNDI"			/* Signature */
	.byte	undiheader_len		/* Length of structure */
	.byte	0			/* Checksum */
	.byte	0			/* Structure revision */
	.byte	0,1,2			/* PXE version: 2.1.0 */
	.word	undiloader		/* Offset to loader routine */
	.word	_data16_memsz		/* Stack segment size */
	.word	_data16_memsz		/* Data segment size */
	.word	_text16_memsz		/* Code segment size */
	.ascii	"PCIR"			/* Bus type */
	.equ undiheader_len, . - undiheader
	.size undiheader, . - undiheader

#ifdef INFOBLK

#define INFOBLK_MAGIC	0xB0079006

infoblk0:				/* PCI function 0 */
infoblk_magic:
	.long	0x00000000		/* Magic = INFOBLK_MAGIC */
infoblk_length:
	.byte	0			/* Length */
	.byte	0			/* Version */
	.byte	0			/* Checksum */
	.byte	0
infoblk_mac_addr:
	.byte	0, 0, 0, 0, 0, 0, 0, 0	/* Mac address */
infoblk_boot_type:
	.byte	0			/* SF_BOOT_TYPE enumeration */
infoblk_mpio_priority:	
	.byte	0			/* 0 = highest */
infoblk_banner_delay:
	.byte	0			/* Period of gPXE banner */
infoblk_bootskip_delay:
	.byte	0			/* Period of 'Press ESC to cancel' banner */
infoblk_boot_image:
	/* Boot image exposed by MC; default 0 = OptionROM+UEFI */
	.byte	0
	.equ	infoblk_len, . - infoblk0
	.size	infoblk0, . - infoblk0
infoblk1:				/* Remaining infoblocks */
	.space	infoblk_len * (num_infoblks - 1), 0x0
	.size	infoblk1, . - infoblk1
#endif

/* This memory, and the functions below it, are used to conditionalise init
 * behaviour. If we have been called as a CLP entry point we skip things like
 * printing messages and waiting for keypresses.
 * The definitions after clp_info must match with the clp_info structure
 * defined in clp.c.
 */
clp_boot:
	.long	0x00000000
	.size	clp_boot, . - clp_boot
clp_info:
clp_es:
	.word	0x0000
clp_edi:
	.long	0x00000000
clp_ds:
	.word	0x0000
clp_esi:
	.long	0x00000000
clp_status:
	.long	0x00000000
	.size	clp_info, . - clp_info

set_clp_boot:
	movl	$clp_boot, clp_boot
	ret
	.size	set_clp_boot, . - set_clp_boot
test_clp_boot:
	cmpl	$clp_boot, clp_boot
	ret
	.size	test_clp_boot, . - test_clp_boot
clear_clp_boot:
	movl	$0, clp_boot
	ret
	.size	clear_clp_boot, . - clear_clp_boot

/* This entry point is referred to in the PCI header, and is called by supported
 * BIOSes to get or override MAC addresses, one-time PXE booting enabling, reset
 * the interface to default settings, etc. if needed.
 *
 * It is called in big real mode, with PMM available, and is run before init.
 */
clp_entry:
	pushf

	/* We need to have %ds = %cs to be able to store the CLP info, but
	 * %ds is also needed for later, so store it in %gs.
	 */
	pushw	%gs
	pushw	%ds
	popw	%gs

	pushw	%ds
	pushw	%cs
	popw	%ds

	call	set_clp_boot

	/* Store CLP call's pointers and segments in memory for later */
	movw	%es, clp_es
	movl	%edi, clp_edi
	movw	%gs, clp_ds
	movl	%esi, clp_esi
	
	popw	%ds
	popw	%gs

	call	init	/* Return with clp_status set */
	pushw	%ds
	pushw	%cs
	popw	%ds
	movl	clp_status, %eax
	popw	%ds
	popf
	lret
	.size	clp_entry, . - clp_entry

clp_exec:
	xorl	%ebp, %ebp	/* Inhibit relocation during POST */
	movl	$c_clp_entry, %eax
	call	exec

	call	clear_clp_boot

	popw	%gs
	popw	%fs
	popw	%es
	popw	%ds
	popaw
	popf
	ret
	.size	clp_exec, . - clp_exec

/* This memory is used to pass information between the prefix and the driver
 * for HP-specific OEM features.
 */
#define HP_BOOT_TYPE_UNSET 0xffff
hp_boot:
	.long	0x00000000
	.size	hp_boot, . - hp_boot
hp_info:
hp_boot_type:
	.word	HP_BOOT_TYPE_UNSET
	.size	hp_info, . - hp_info
set_hp_boot:
	movl	$hp_boot, hp_boot
	ret
	.size	set_hp_boot, . - set_hp_boot
	.globl	test_hp_boot
test_hp_boot:
	cmpl	$hp_boot, hp_boot
	ret
	.size	test_hp_boot, . - test_hp_boot

/* Initialisation
 * Normally called once during POST. May be called additional times from other
 * entry points that use this init code.
 *
 * Determine whether or not this is a PnP system via a signature
 * check.  If it is PnP, return to the PnP BIOS indicating that we are
 * a boot-capable device; the BIOS will call our boot execution vector
 * if it wants to boot us.  If it is not PnP, hook INT 19.
 */
init:
	/* Preserve flags, registers, clear direction flag, set %ds=%cs */
	pushf
	pushaw
	pushw	%ds
	pushw	%es
	pushw	%fs
	pushw	%gs
	cld
	pushw	%cs
	popw	%ds

	/* Shuffle some registers around.  We need %di available for
	 * the print_xxx functions, and in a register that's
	 * addressable from %es, so shuffle as follows:
	 *
	 *    %di (pointer to PnP structure) => %bx
	 *    %bx (runtime segment address, for PCI 3.0) => %gs
	 */
	movw	%bx, %gs
	movw	%di, %bx

	/* Store PCI bus:dev.fn address */
	movw	%ax, init_pci_busdevfn

	call	test_clp_boot
	je	5f

	/* Print message as early as possible */
	movw	$init_message, %si
	xorw	%di, %di
	call	print_message
	call	print_pci_busdevfn

5:
	/* Save PCI bus:dev.fn information */
	pushl	%ebx
	movzwl	%ax, %ebx
	movl	%ebx, ( pci_busdevfn )
	popl	%ebx

	call	test_clp_boot
	je	pnp_done

	/* Print segment address */
	movb	$' ', %al
	xorw	%di, %di
	call	print_character
	movw	%cs, %ax
	call	print_hex_word

	/* Check for PCI BIOS version */
	pushl	%ebx
	pushl	%edx
	pushl	%edi
	stc
	movw	$0xb101, %ax
	int	$0x1a
	jc	no_pci3
	cmpl	$PCI_SIGNATURE, %edx
	jne	no_pci3
	testb	%ah, %ah
	jnz	no_pci3
	movw	$init_message_pci, %si
	xorw	%di, %di
	call	print_message
	movb	%bh, %al
	call	print_hex_nibble
	movb	$'.', %al
	call	print_character
	movb	%bl, %al
	call	print_hex_byte
	cmpb	$3, %bh
	jb	no_pci3
	/* PCI >=3.0: leave %gs as-is if sane */
	movw	%gs, %ax
	cmpw	$0xa000, %ax	/* Insane if %gs < 0xa000 */
	jb	pci3_insane
	movw	%cs, %bx	/* Sane if %cs == %gs */
	cmpw	%bx, %ax
	je	1f
	movzbw	romheader_size, %cx /* Sane if %cs+len <= %gs */
	shlw	$5, %cx
	addw	%cx, %bx
	cmpw	%bx, %ax
	jae	1f
	movw	%cs, %bx	/* Sane if %gs+len <= %cs */
	addw	%cx, %ax
	cmpw	%bx, %ax
	jbe	1f
pci3_insane: /* PCI 3.0 with insane %gs value: print error and ignore %gs */
	movb	$'!', %al
	call	print_character
	movw	%gs, %ax
	call	print_hex_word
no_pci3:
	/* PCI <3.0: set %gs (runtime segment) = %cs (init-time segment) */
	pushw	%cs
	popw	%gs
1:	popl	%edi
	popl	%edx
	popl	%ebx

	/* Check for PnP BIOS.  Although %es:di should point to the
	 * PnP BIOS signature on entry, some BIOSes fail to do this.
	 */
	movw	$( 0xf000 - 1 ), %bx
pnp_scan:
	incw	%bx
	jz	no_pnp
	movw	%bx, %es
	cmpl	$PNP_SIGNATURE, %es:0
	jne	pnp_scan
	xorw	%dx, %dx
	xorw	%si, %si
	movzbw	%es:5, %cx
1:     	es lodsb
	addb	%al, %dl
	loop	1b
	jnz	pnp_scan
pnp:    /* Is PnP: print PnP message */
	movw	$init_message_pnp, %si
	xorw	%di, %di
	call	print_message
	jmp	pnp_done
no_pnp:		/* Not PnP-compliant - hook INT 19 */
#ifdef NONPNP_HOOK_INT19
	movw	$init_message_int19, %si
	xorw	%di, %di
	call	print_message
	xorw	%ax, %ax
	movw	%ax, %es
	pushl	%es:( 0x19 * 4 )
	popl	orig_int19
	pushw	%gs /* %gs contains runtime %cs */
	pushw	$int19_entry
	popl	%es:( 0x19 * 4 )
#endif /* NONPNP_HOOK_INT19 */
pnp_done:

	/* Check for PMM */
	movw	$( 0xe000 - 1 ), %bx
pmm_scan:
	incw	%bx
	jz	no_pmm
	movw	%bx, %es
	cmpl	$PMM_SIGNATURE, %es:0
	jne	pmm_scan
	xorw	%dx, %dx
	xorw	%si, %si
	movzbw	%es:5, %cx
1:	es lodsb
	addb	%al, %dl
	loop	1b
	jnz	pmm_scan

	call	test_clp_boot
	je	5f

	/* PMM found: print PMM message */
	movw	$init_message_pmm, %si
	xorw	%di, %di
	call	print_message

5:
	/* We have PMM and so a 1kB stack: preserve whole registers */
	pushal
	/* Allocate image source PMM block.  Round up the size to the
	 * nearest 4kB (8 512-byte sectors) to work around AMI BIOS bugs.
	 */
	movzbl	romheader_size, %ecx
	addw	extra_size, %cx
	addw	$0x0007, %cx	/* Round up to multiple of 8 512-byte sectors */
	andw	$0xfff8, %cx
	shll	$5, %ecx
	movl	$PMM_HANDLE_BASE_IMAGE_SOURCE, %ebx
	movw	$get_pmm_image_source, %bp
	call	get_pmm
	movl	%esi, image_source
	jz	1f
	/* Copy ROM to image source PMM block */
	pushw	%es
	xorw	%ax, %ax
	movw	%ax, %es
	movl	%esi, %edi
	xorl	%esi, %esi
	movzbl	romheader_size, %ecx
	shll	$7, %ecx
	addr32 rep movsl	/* PMM presence implies flat real mode */
	popw	%es
	/* Shrink ROM */
	movw	pciheader_runtime_length, %ax
	movb	%al, romheader_size
1:	/* Allocate decompression PMM block.  Round up the size to the
	 * nearest 128kB and use the size within the PMM handle; this
	 * allows the same decompression area to be shared between
	 * multiple iPXE ROMs even with differing build IDs
	 */
	movl	$_textdata_memsz_pgh, %ecx
	addl	$0x00001fff, %ecx
	andl	$0xffffe000, %ecx
	movl	%ecx, %ebx
	shrw	$12, %bx
	orl	$PMM_HANDLE_BASE_DECOMPRESS_TO, %ebx
	movw	$get_pmm_decompress_to, %bp
	call	get_pmm
	movl	%esi, decompress_to

	/* Restore registers */
	popal

	call	test_clp_boot
	je	clp_exec

no_pmm:
	/* PCI3.0 requires we copy ourselves to option ROM space in %gs.
	 * This is now done later (after updating prodstr), but print
	 * out the destination now */
	movb	$' ', %al
	xorw	%di, %di
	call	print_character
	movw	%gs, %ax
	call	print_hex_word
	movb	$'\n', %al
	call	print_character

	/* Write out prodstr */
	movl	( pci_busdevfn ), %eax
	movw	$prodstr_pci_id, %di
	call	print_pci_busdevfn
	xorw	%di, %di

	/* The firmware checks the subsystem vendor id to ensure that
	 * HP-specific features are not enabled on non HP NICs.
	 * Here we do this check to avoid loading the driver if we don't have
	 * to.
	 *
	 * Check the subsystem vendor id.
	 * Instructions details available at
	 * http://www.delorie.com/djgpp/doc/rbinter/id/86/23.html
	 */
	movw	$0xb109, %ax
	movw	( pci_busdevfn ), %bx
	movw	$0x2c, %di
	int	$0x1a
	jc	skip_hp_feat	/* int call wasn't successfull. */
	cmpw	$0x103C, %cx	/* Compare with HP subsystem vendor id. */
	jne	skip_hp_feat

	call	set_hp_boot
	xorl	%ebp, %ebp	/* Inhibit relocation during POST */
	movl	$hp_features, %eax
	call	exec
skip_hp_feat:

#ifdef DEBUG
	/* Print the BAR register we use */
	movw	$init_message_bar, %si
	xorw	%di, %di
	call	print_message
	call	get_pci_bar
	movw	%di, %ax
	xorw	%di, %di
	call	print_hex_word
	movb	$'\n', %al
	call	print_character
#endif

#ifdef INFOBLK
	/* Attempt to read the INFOBLK. Use the pci function to select
	 * the appropriate one (and hope nobody uses pass-thru) */
	movl	( pci_busdevfn ), %eax
	andw	$0xff, %ax
	xorw	%dx, %dx
	movw	$num_infoblks, %bx
	divw	%bx, %ax
	movw	%dx, %bx		/* dx is remainder from division */
	imulw	$infoblk_len, %bx
	cmpl	$INFOBLK_MAGIC, infoblk_magic(%bx)
	jne	no_infoblk
#ifdef DEBUG
	movb	$'1', %al
	call	print_character
#endif
	cmpb	$infoblk_len, infoblk_length(%bx)
	jb	no_infoblk
#ifdef DEBUG
	movb	init_pci_busdevfn, %al
	andb	$PCI_FUNC_MASK, %al
	addb	$'0', %al
	call	print_character
#endif

	/* Skip prompt if this is not the first PCI function */
	testb	$PCI_FUNC_MASK, init_pci_busdevfn
	jnz	no_shell

#ifdef UEFI_CHECK
	/* Preserve infoblk pointer */
	pushw	%bx

	/* If the EFI driver has loaded, suppress the Ctrl+B prompt.  When the
	 * EFI driver loads, it writes 0x5FC into ADR_REGION3 of the address
	 * region register.  Read the value in ADR_REGION3 and if it is 0x5FC,
	 * set it to zero and skip the printing of init_message_prompt.
	 *
	 * Code in mromprefix.S has been adapted to find the address in BAR 2.
	 */
	movw	init_pci_busdevfn, %bx
	call	get_pci_bar
	pushl	%ebx
	movw	$PCIBIOS_READ_CONFIG_DWORD, %ax
	int	$0x1a			/* Read the BAR value */
	popl	%ebx
	popw	%bx
	xorw	%di, %di
	movl	%ecx, %edx
	andl	$0xfffffff0, %edx	/* Clear lower four bits */
	addl	$0xc, %edx
	xorw	%ax, %ax
	movw	%ax, %es
	movl	%es:(%edx), %eax
	cmpl	$0x5fc, %eax
	jne	2f
	/* Zero all of ADR_REGION_REG, as is the case after a reset */
	xorl	%eax, %eax
	subl	$0xc, %edx
	movl	$4, %ecx
1:	movl	%eax, %es:(%edx)
	addl	$4, %edx
	loop	1b
	movw	$init_message_skip_prompt, %si
	call	print_message
	jmp	no_shell
2:
	/* Suppress prompt if the system supports UEFI and boot-image=all */
	call	check_smbios_uefi
	jz	3f
	cmpb	$0, infoblk_boot_image
	jz	no_shell
#else
	xorw	%di, %di
#endif

3:	/* Prompt for POST-time shell */
	pushw	%bx
	movw	$init_message_prompt, %si
	call	print_message
 	movw	$prodstr, %si
 	call	print_message
	/* Wait for Ctrl-B */
	movzbw	infoblk_banner_delay(%bx), %cx
	imulw	$18, %cx
	movw	$0xff02, %bx
	call	wait_for_key
	/* Clear prompt */
	pushf
	call	print_kill_line
	movw	$init_message_done, %si
	call	print_message
	popf
	/* Restore infoblk pointer */
	popw	%bx
	/* If Ctrl-B was pressed: invoke gPXE, and give up using
	 * the infoblk - gPXE will return all the information we
	 * need anyway.
	 */
	jz	no_infoblk
no_shell:
#ifdef DEBUG
	movb	$'n', %al
	call	print_character
#endif
	/* Complete rest of prodstr */
	pushw	%cs
	popw	%es
	movw	$infoblk_mac_addr, %si
	addw	%bx, %si
#ifdef BCV
	movw	$prodstr_mac_addr, %di
	call	print_mac_addr
#endif
	movb	$' ', prodstr_separator
	xorw	%di, %di

	/* Validate boot_type */
	movzbw	infoblk_boot_type(%bx), %cx

	call	test_hp_boot
	jne	5f

	cmpw	$HP_BOOT_TYPE_UNSET, hp_boot_type
	je	5f
	movw	hp_boot_type, %cx
5:
#ifdef BCV_PROTO_ISCSI
	cmpw	$SF_BOOT_MODE_DISABLE, %cx
	jbe	2f
	movw	$SF_BOOT_MODE_DISABLE, %cx
2:	/* Squash iSCSI boot if this isn't mpio_priority=1 */
	cmpw	$SF_BOOT_MODE_BCV, %cx
	jne	3f
	cmpb	$1, infoblk_mpio_priority(%bx)
	jbe	3f
	movw	$SF_BOOT_MODE_DISABLE, %cx
	/* Don't print the current boot mode */
	jmp	4f
#else
	cmpw	$SF_BOOT_MODE_BCV, %cx
	jbe	3f
	movw	$SF_BOOT_MODE_DISABLE, %cx
#endif
3:	/* Give the user the opportunity to abort */
	movzbw	infoblk_bootskip_delay(%bx), %dx
	imulw	$18, %dx
	call	request_abort
	jmp	4f

no_infoblk:
	/* Prompt for POST-time shell, determine boot mode, and request abort */
	xorl	%ebp, %ebp	/* Inhibit relocation during POST */
	movl	$instance_init_select_mode, %eax
	call	exec
4:
#elif defined(BCV)
	/* Prompt for POST-time shell, determine boot mode, and request abort */
	xorl	%ebp, %ebp	/* Inhibit relocation during POST */
	movl	$instance_init_select_mode, %eax
	call	exec
#else
	/* Skip prompt if this is not the first PCI function */
	testb	$PCI_FUNC_MASK, init_pci_busdevfn
	jnz	no_shell
	/* Prompt for POST-time shell */
	movw	$init_message_prompt, %si
	call	print_message
	movb	$' ', prodstr_separator
	movw	$prodstr, %si
	call	print_message
	movw	$init_message_dots, %si
	call	print_message
	/* Wait for Ctrl-B */
	movw	$ROM_BANNER_TIMEOUT, %cx
	movw	$0xff02, %bx
	call	wait_for_key
	/* Clear prompt */
	pushf
	call	print_kill_line
	movw	$init_message_done, %si
	call	print_message
	popf
	jnz	no_shell
	/* Ctrl-B was pressed: invoke gPXE.  The keypress will be
	 * picked up by the initial shell prompt, and we will drop
	 * into a shell.
	 */
	xorl	%ebp, %ebp	/* Inhibit relocation during POST */
	movl	$instance_main, %eax
	call	exec
no_shell:
	movb	$'\n', %al
	xorw	%di, %di
	call	print_character

	/* Give the user the opportunity to abort */
	movw	$SF_BOOT_MODE_BEV, %cx
	movw	$ROM_BANNER_TIMEOUT, %dx
	call	request_abort
#endif

#ifdef BCV
	cmpw	$SF_BOOT_MODE_BCV, %cx
	jne	2f
	/* iSCSI boot mode: update $PnP header to indicate that we are a
	 * SCSI device with a BCV, rather than a network card with a BEV.
	 * We need to update the $PnP header (including checksum). The
	 * checksum will be updated later
	 */
	movb	$1, pnpheader_devtype
	movb	$0xc4, pnpheader_devind
	movw	$bcv_entry, pnpheader_bcv
	movw	$0, pnpheader_bev
	/* Fix up the PnP header checksum */
	xorw	%bx, %bx
	movw	$pnpheader, %si
	movw	$pnpheader_len, %cx
1:	lodsb
	addb	%al, %bl
	loop	1b
	subb	%bl, pnpheader_checksum
	/* Unhook INT19 if necessary */
	call	unhook_int19
	jmp	3f
2:
#endif
	cmpw	$SF_BOOT_MODE_DISABLE, %cx
	jne	3f
	/* Disable boot completely: shrink the ROM size to 0 */
	movb	$0, romheader_size
	/* Unhook INT19 if necessary */
	call	unhook_int19
3:	/* Update the romheader checksum */
	xorw	%bx, %bx
	xorw	%si, %si
	movzbw	romheader_size, %cx
	shlw	$9, %cx
	pushw	%cx
4:	lodsb
	addb	%al, %bl
	loop	4b
	subb	%bl, checksum
	popw	%cx
	/* Copy ourselves to the PCI3.0 option ROM space. Will be a no-op
	 * for lower PCI versions.
	 */
	pushw	%gs
	popw	%es
	xorw	%si, %si
	xorw	%di, %di
	cs rep	movsb

	/* Restore registers */
	popw	%gs
	popw	%fs
	popw	%es
	popw	%ds
	popaw

#ifdef BCV
	/* Indicate INT13 device capability to PnP BIOS, if present */
	movw	$0x110, %ax
	cmpw	$0, %cs:pnpheader_bev
	je	1f
#endif
	/* Indicate boot capability to PnP BIOS if present */
	movw	$0x20, %ax
	cmpb	$0, %cs:romheader_size
	jne	1f
	movw	$0x0, %ax
1:	/* Restore flags */
	popf
	lret
	.size init, . - init

/* Attempt to find or allocate PMM block
 *
 * Parameters:
 *  %ecx : size of block to allocate, in paragraphs
 *  %ebx : PMM handle base
 *  %bp : routine to check acceptability of found blocks
 *  %es:0000 : PMM structure
 * Returns:
 *  %ebx : PMM handle
 *  %esi : allocated block address, or zero (with ZF set) if allocation failed
 */
get_pmm:
	/* Preserve registers */
	pushl	%eax
	pushw	%di
	movw	$' ', %di
get_pmm_find:
	/* Try to find existing block */
	pushl	%ebx		/* PMM handle */
	pushw	$PMM_FIND
	lcall	*%es:7
	addw	$6, %sp
	pushw	%dx
	pushw	%ax
	popl	%esi
	/* Treat 0xffffffff (not supported) as 0x00000000 (not found) */
	incl	%esi
	jz	get_pmm_allocate
	decl	%esi
	jz	get_pmm_allocate
	/* Block found - check acceptability */
	call	*%bp
	jnc	get_pmm_done
	/* Block not acceptable - increment handle and retry */
	incl	%ebx
	jmp	get_pmm_find
get_pmm_allocate:
	/* Block not found - try to allocate new block */
	pushw	$0x0002		/* Extended memory */
	pushl	%ebx		/* PMM handle */
	pushl	%ecx		/* Length */
	pushw	$PMM_ALLOCATE
	lcall	*%es:7
	addw	$12, %sp
	pushw	%dx
	pushw	%ax
	popl	%esi
	/* Check if PMM area is writable - if not, set %esi to zero */
	call	get_pmm_validate
	movw	$'+', %di	/* Indicate allocation attempt */
get_pmm_done:
	call	test_clp_boot
	je	5f
	/* Print block address */
	movw	%di, %ax
	xorw	%di, %di
	call	print_character
	movl	%esi, %eax
	call	print_hex_dword
5:
	/* Treat 0xffffffff (not supported) as 0x00000000 (allocation
	 * failed), and set ZF to indicate a zero result.
	 */
	incl	%esi
	jz	1f
	decl	%esi
1:	/* Restore registers and return */
	popw	%di
	popl	%eax
	ret
	.size	get_pmm, . - get_pmm

	/* Check acceptability of image source block */
get_pmm_image_source:
	pushw	%es
	xorw	%ax, %ax
	movw	%ax, %es
	movl	build_id, %eax
	addr32 cmpl %es:build_id(%esi), %eax
	je	1f
	stc
1:	popw	%es
	ret
	.size	get_pmm_image_source, . - get_pmm_image_source

	/* Check acceptability of decompression block */
get_pmm_decompress_to:
	clc
	ret
	.size	get_pmm_decompress_to, . - get_pmm_decompress_to

	/* Check if the PMM allocated memory area is writable.
	 *
	 * Write the value 0x1234fedc to the start of the PMM allocated area
	 * and read this address back.  If the correct value is not read back,
	 * assume that something is wrong with this PMM allocation and set
	 * %esi to zero to indicate this.  The initialisation and execution
	 * functions will continue as though this PMM allocation failed. */
get_pmm_validate:
	pushl	%eax
	pushw	%es
	testl	%esi, %esi
	jz	1f			/* Don't try writing to address 0x0 */
	xorw	%ax, %ax
	movw	%ax, %es
	movl	$0x1234fedc, %es:(%esi)
	movl	%es:(%esi), %eax
	cmpl	$0x1234fedc, %eax
	je	1f
	xorl	%esi, %esi
1:	popw	%es
	popl	%eax
	ret
	.size	get_pmm_validate, . - get_pmm_validate

/*****************************************************************************
 * check_smbios_uefi
 *
 * Read the SMBIOS table to determine if UEFI is supported.
 *
 * There is a single Structure table which contains structures of different
 * types.  Each structure has a formatted length, then an arbitrary number
 * of strings, and finally is terminated by 0x0000.
 *
 * The "UEFI-supported" bit is in structure type 0, offset 0x13, bit 3.
 *
 * Parameters:
 *   None
 * Returns:
 *   ZF clear if UEFI is supported
 * Corrupts:
 *   None
 *****************************************************************************
 */
check_smbios_uefi:
	pushw	%es
	pushl	%eax
	pushl	%ebx
	pushl	%edx

	/* Search from 0xF0000 to 0xFFFFF for signature (16-byte aligned) */
	movw	$ ( 0xf000 - 1 ), %ax
1:	incw	%ax
	jz	smbios_done
	movw	%ax, %es
	cmpl	$SMBIOS_SIGNATURE, %es:0
	jne	1b
	/* Calculate checksum over entry point structure */
	pushw	%ax
	pushw	%cx
	pushw	%si
	movzbw	%es:0x5, %cx		/* Length of entry point structure */
	xorw	%si, %si
	xorb	%bl, %bl
2:	es lodsb
	addb	%al, %bl
	loop	2b
	popw	%si
	popw	%cx
	popw	%ax
	jnz	1b			/* ZF set/cleared by ADD in loop */
smbios_found:
	movzwl	%es:0x16, %edx		/* Structure table length */
	movl	%es:0x18, %eax		/* Structure table address */
	addl	%eax, %edx		/* End address of Structure table */
	xorl	%ebx, %ebx
	movw	%bx, %es
smbios_find_type0:
	movb	%es:(%eax), %bl
	testb	%bl, %bl
	jz	smbios_type0_found
	movb	%es:1(%eax), %bl	/* Length of formatted section */
	addl	%ebx, %eax
	decl	%eax
1:	incl	%eax
	cmpl	%edx, %eax		/* Check for end of all structures */
	jge	smbios_type0_fail
	movw	%es:(%eax), %bx		/* WORD 0x0000 terminates strings */
	testw	%bx, %bx
	jnz	1b
	addl	$0x2, %eax		/* 2 bytes for WORD comparison */
	jmp	smbios_find_type0

smbios_type0_found:
	/* BIOS Characteristics Extension Byte #2 at offset 0x13 */
	addl	$0x13, %eax
	movb	%es:(%eax), %bl
	andb	$0x8, %bl		/* Bit 3 is set if UEFI is supported */
	/* ZF set/cleared by AND */
smbios_done:
	/* Restore registers and return */
	popl	%edx
	popl	%ebx
	popl	%eax
	popw	%es
	ret

smbios_type0_fail:
	/* Couldn't find type 0 structure; set ZF and finish */
	cmpb	%al, %al
	jmp	smbios_done

/* Find the PCI BAR to use
 *
 * This looks at BAR 0. If that is an I/O BAR we have to use BAR 2.
 *
 * Parameters:
 *   None
 * Returns:
 *   %di : PCI BAR
 * Corrupts:
 *   %ax
 */
get_pci_bar:
	pushw	%bx
	pushw	%cx
	movw	init_pci_busdevfn, %bx
	movw	$PCI_BAR_0, %di
	/* Issue INT 0x1a,b10a */
	movw	$PCIBIOS_READ_CONFIG_DWORD, %ax
	int	$0x1a
	jc	3f		/* Fall back on failure */
	testb	$0x01, %cl	/* Skip the I/O BAR */
	jz	99f
3:	movw	$PCI_BAR_2, %di
99:	popw	%cx
	popw	%bx
	ret

/*
 * Note to hardware vendors:
 *
 * If you wish to brand this boot ROM, please do so by defining the
 * strings PRODUCT_NAME and PRODUCT_SHORT_NAME in config/general.h.
 *
 * While nothing in the GPL prevents you from removing all references
 * to gPXE or http://etherboot.org, we prefer you not to do so.
 *
 * If you have an OEM-mandated branding requirement that cannot be
 * satisfied simply by defining PRODUCT_NAME and PRODUCT_SHORT_NAME,
 * please contact us.
 *
 * [ Including an ASCII NUL in PRODUCT_NAME is considered to be
 *   bypassing the spirit of this request! ]
 */
init_message:
	.ascii	"\n"
	.ascii	PRODUCT_NAME
	.ascii	"\n"
	.ascii	PRODUCT_AUTHOR
	.ascii	"\n"
	.asciz	"gPXE (http://etherboot.org) - "
	.size	init_message, . - init_message
init_message_pci:
	.asciz	" PCI"
	.size	init_message_pci, . - init_message_pci
init_message_pnp:
	.asciz	" PnP"
	.size	init_message_pnp, . - init_message_pnp
init_message_pmm:
	.asciz	" PMM"
	.size	init_message_pmm, . - init_message_pmm
#ifdef DEBUG
init_message_bar:
	.asciz	" BAR "
	.size	init_message_bar, . - init_message_bar
#endif
init_message_int19:
	.asciz	" INT19"
	.size	init_message_int19, . - init_message_int19
#if defined(INFOBLK) || !defined(BCV)
init_message_prompt:
	.asciz	"Press Ctrl-B to configure "
	.size	init_message_prompt, . - init_message_prompt
#endif
init_message_skip_prompt:
	.asciz	"UEFI driver already loaded. Skipping configuration prompt.\n"
	.size	init_message_skip_prompt, . - init_message_skip_prompt
init_message_abort_dots:
	.ascii	" boot"
	.size	init_message_abort_dots, . - init_message_abort_dots
init_message_dots:
	.asciz	"..."
	.size	init_message_dots, . - init_message_dots
init_message_done:
	.asciz	"\n\n"
	.size	init_message_done, . - init_message_done
init_message_mode:
	.asciz	"Configured boot mode: "
	.size	init_message_mode, . - init_message_mode
init_message_pxe:
	.asciz	"PXE"
	.size	init_message_pxe, . - init_message_pxe
#ifdef BCV_PROTO_ISCSI
init_message_iscsi:
	.asciz	"iSCSI"
	.size	init_message_iscsi, . - init_message_iscsi
#endif
init_message_disable:
	.asciz	"disabled"
	.size	init_message_disable, . - init_message_disable
init_message_abort:
	.asciz	"Press ESC to skip "
	.size	init_message_abort, . - init_message_abort
init_message_aborted:
	.asciz	" boot disabled."
	.size	init_message_aborted, . - init_message_aborted

/* PCI bus:dev.fn
 *
 * If upper 2 bytes are 0xffff, data was never written
 */
pci_busdevfn:
	.long	0xffffffff
	.size	pci_busdevfn, . - pci_busdevfn

/* PCI bus:dev.fn
 *
 */
init_pci_busdevfn:
	.word	0xffff
	.size	init_pci_busdevfn, . - init_pci_busdevfn

/* Image source area
 *
 * May be either zero (indicating to use option ROM space as source),
 * or within a PMM-allocated block.
 */
image_source:
	.long	0
	.size	image_source, . - image_source

/* Additional image source size (in 512-byte sectors)
 *
 */
extra_size:
	.word	0
	.size	extra_size, . - extra_size

/* Temporary decompression area
 *
 * May be either zero (indicating to use default decompression area in
 * high memory), or within a PMM-allocated block
 */
decompress_to:
	.long	0
	.size	decompress_to, . - decompress_to

#ifdef BCV
/* Boot Connection Vector entry point
 *
 * Called by the PnP BIOS to allow us to hook SCSI devices.
 */
bcv_entry:
	pushfl
	pushal
	pushw	%gs
	pushw	%fs
	pushw	%es
	pushw	%ds
	cld
	orl	$0xffffffff, %ebp	/* Allow arbitrary relocation */
	movl	$instance_bcv_main, %eax
	call	exec
	popw	%ds
	popw	%es
	popw	%fs
	popw	%gs
	popal
	popfl
	lret
	.size	bcv_entry, . - bcv_entry
#endif

/* Boot Execution Vector entry point
 *
 * Called by the PnP BIOS when it wants to boot us.
 */
bev_entry:
	orl	$0xffffffff, %ebp	/* Allow arbitrary relocation */
	movl	$instance_main, %eax
	pushal
	pushw	%ds
	call	exec
	popw	%ds
	popal
	lret
	.size	bev_entry, . - bev_entry

/* INT19 entry point
 *
 * Called via the hooked INT 19 if we detected a non-PnP BIOS.  We
 * attempt to return via the original INT 19 vector (if we were able
 * to store it).
 */
int19_entry:
	pushw	%cs
	popw	%ds
	/* Prompt user to press B to boot */
	movw	$int19_message_prompt, %si
	xorw	%di, %di
	call	print_message
	movw	$prodstr, %si
	call	print_message
	movw	$init_message_dots, %si
	call	print_message
	movw	$0xdf4e, %bx
	movw	$ROM_BANNER_TIMEOUT, %cx
	call	wait_for_key
	pushf
	xorw	%di, %di
	call	print_kill_line
	movw	$init_message_done, %si
	call	print_message
	popf
	jz	1f
	/* Leave keypress in buffer and start gPXE.  The keypress will
	 * cause the usual initial Ctrl-B prompt to be skipped.
	 */
	orl	$0xffffffff, %ebp	/* Allow arbitrary relocation */
	movl	$instance_main, %eax
	call	exec
1:	/* Try to call original INT 19 vector */
	movl	%cs:orig_int19, %eax
	testl	%eax, %eax
	je	2f
	ljmp	*%cs:orig_int19
2:	/* No chained vector: issue INT 18 as a last resort */
	int	$0x18
	.size	int19_entry, . - int19_entry
orig_int19:
	.long	0
	.size	orig_int19, . - orig_int19

int19_message_prompt:
	.asciz	"Press N to skip booting from "
	.size	int19_message_prompt, . - int19_message_prompt

/* Unhook INT19 (if it was ever hooked)
 */
unhook_int19:
	pushl	%eax
	movl	orig_int19, %eax
	cmpl	$0, %eax
	je	1f
	pushw	%bx
	pushw	%es
	xorw	%bx, %bx
	movw	%bx, %es
	movl	%eax, %es:( 0x19 * 4 )
	popw	%es
	popw	%bx
1:	popl	%eax
	ret
	
/* Execute as a boot device
 *
 * Entry: %eax = C routine to invoke
 * Exit:  %ecx = Returned values from C routine if any (or otherwise currupted)
 *        %bp, %esi, %edi, %es, %edx = Corrupted
 */
exec:	/* Set %ds = %cs */
	pushw	%cs
	popw	%ds

	/* Save entry point to a register which is preserved below */
	movl	%eax, %ecx

	call	test_clp_boot
	je	5f

	/* Print message as soon as possible */
	movw	$prodstr, %si
	xorw	%di, %di
	call	print_message
	movw	$exec_message_pre_install, %si
	call	print_message

5:
	/* Install gPXE */
	call	alloc_basemem
	movl	image_source, %esi
	movl	decompress_to, %edi
	call	install_prealloc

	call	test_clp_boot
	je	5f

	/* Print message indicating successful installation */
	movw	$exec_message_post_install, %si
	xorw	%di, %di
	call	print_message

5:
#ifdef BCV
	/* Save entry point on stack for copyout */
	pushl	%ecx
#endif
	/* Prepare to jump back to .prefix */
	pushw	%cs
	pushw	$2f

	/* Store magic word on BIOS stack and remember BIOS %ss:sp */
	pushl	$STACK_MAGIC
	movw	%ss, %dx
	movw	%sp, %bp

	/* Set up real-mode stack */
#if defined(EBDA_ALLOCATE) || defined(EBDA_RELOCATE)
	call	allocate_stack
	movw	%si, %ss
	movw	%di, %sp
#else
	movw	%bx, %ss
	movw	$_estack16, %sp
#endif

	call	test_clp_boot
	jne	5f
	movl	$clp_info, %edi
	jmp	6f
5:	call	test_hp_boot
	jne	6f
	movl	$hp_info, %edi
6:

	/* Jump to .text16 segment */
	pushw	%ax
	pushw	$1f
	lret
	.section ".text16", "awx", @progbits

1:	/* Allow realmode code access to the prefix */
	movw	%ds, %cs:prefix_seg
	/* Call required entry point (with PCI bus/dev/fn in %ecx) */
	pushl	%ecx
	movl	( pci_busdevfn ), %ecx
	pushw	%cs
	call	prot_call
	popl	%edi /* discard */

	/* Restore BIOS stack */
	movw	%dx, %ss
	movw	%bp, %sp

	/* Check magic word on BIOS stack */
	popl	%edi
	cmpl	$STACK_MAGIC, %edi
	je	1f
	/* BIOS stack corrupt: use INT 18 */
	int	$0x18
1:	/* BIOS stack OK: Uninstall from prefix */
	lret
	.previous
2:
#if defined(EBDA_ALLOCATE) || defined(EBDA_RELOCATE)
	call	release_stack
#endif
#ifdef BCV
	/* Check that we're being executed from init */
	popl	%edi
	cmpl	$instance_init_select_mode, %edi
	jne	3f
	/* Copy the mac address out before destroying text16 */
	movw	%bx, %es
	movw	$instance_mac_addr, %si
	movw	$prodstr_mac_addr, %di
	call	print_mac_addr
	movb	$' ', prodstr_separator
	/* Give the user the option of aborting */
	movzbw	%es:bootskip_delay, %dx
	imulw	$18, %dx
	call	request_abort
3:
#endif
	call	uninstall
	ret

exec_message_pre_install:
	.asciz	" starting execution..."
	.size exec_message_pre_install, . - exec_message_pre_install
exec_message_post_install:
	.asciz	"ok\n"
	.size exec_message_post_install, . - exec_message_post_install

/* UNDI loader
 *
 * Called by an external program to load our PXE stack.
 */
undiloader:
	/* Save registers */
	pushl	%esi
	pushl	%edi
	pushw	%ds
	pushw	%es
	pushw	%bx
	/* ROM segment address to %ds */
	pushw	%cs
	popw	%ds
	/* UNDI loader parameter structure address into %es:%di */
	movw	%sp, %bx
	movw	%ss:18(%bx), %di
	movw	%ss:20(%bx), %es
	/* Install to specified real-mode addresses */
	pushw	%di
	movw	%es:12(%di), %bx
	movw	%es:14(%di), %ax
	movl	image_source, %esi
	movl	decompress_to, %edi
	orl	$0xffffffff, %ebp	/* Allow arbitrary relocation */
	call	install_prealloc
	popw	%di
	/* Call UNDI loader C code */
	pushl	$pxe_loader_call
	pushw	%cs
	pushw	$1f
	pushw	%ax
	pushw	$prot_call
	lret
1:	popw	%bx	/* discard */
	popw	%bx	/* discard */
	/* Restore registers and return */
	popw	%bx
	popw	%es
	popw	%ds
	popl	%edi
	popl	%esi
	lret
	.size undiloader, . - undiloader

/* Table of message strings indexed by boot type */
init_message_boot_vector:
	.word	init_message_pxe
#ifdef BCV_PROTO_ISCSI
	.word	init_message_iscsi
#else
	.word	init_message_disable
#endif
	.word	init_message_disable
	.size	init_message_boot_vector, . - init_message_boot_vector

/* Print the boot mode in %cx. Corrupts %si (and maybe %di) */
print_boot_mode:
	pushw	%di
	movw	%cx, %di
	shlw	$1, %di
	movw	init_message_boot_vector(%di), %si
	popw	%di
	call	print_message
	ret

/* Give the user the option of aborting the configured boot mode. Near-call
 *
 * Input:
 *   %cx:      Current boot mode
 *   %dx:      Number of ticks to wait for
 * Output:
 *   %cx:      New boot mode
 * Corrupts:
 *   flags, %si, %di
 */
request_abort:
	/* Preserve registers */
	pushw	%bx
	pushw	%ax
	xorw	%di, %di
	/* Current boot mode */
	movw	$init_message_mode, %si
	call	print_message
	call	print_boot_mode
	movb	$'\n', %al
	call	print_character
	/* Check if abort message has been disabled */
	cmp	$0, %dx
	je	99f
	/* Check if abort message is appropriate */
	cmpw	$SF_BOOT_MODE_DISABLE, %cx
	je	99f
	/* Print abort message */
	movw	$init_message_abort, %si
	call	print_message
	call	print_boot_mode
	movw	$init_message_abort_dots, %si
	call	print_message
	/* Wait for ESC */
	pushw	%cx
	movw	%dx, %cx
	movw	$0xff1b, %bx
	call	wait_for_key
	popw	%cx
	jnz	98f
	/* Abort */
	call	print_boot_mode
	movw	$init_message_aborted, %si
	call	print_message
	movw	$SF_BOOT_MODE_DISABLE, %cx
98:	movb	$'\n', %al
	call	print_character
	call	print_character
99:	/* Restore registers */
	popw	%ax
	popw	%bx
	ret

/* Wait for key press specified by %bl (masked by %bh)
 * Will wait for the number of ticks in %cx. %cx is corrupted
 *
 * Used by init and INT19 code when prompting user.  If the specified
 * key is pressed, it is left in the keyboard buffer.
 *
 * Returns with ZF set iff specified key is pressed.
 */
wait_for_key:
	/* Preserve registers */
	pushw	%ax
1:	/* Empty the keyboard buffer before waiting for input */
	movb	$0x11, %ah
	int	$0x16
	jz	2f
	movb	$0x10, %ah
	int	$0x16
	jmp	1b
2:	/* Wait for a key press */
3:	decw	%cx
	js	99f		/* Exit with ZF clear */
	/* Wait for timer tick to be updated */
	call	wait_for_tick
	/* Check to see if a key was pressed */
	movb	$0x11, %ah
	int	$0x16
	jz	3b
	/* Check to see if key was the specified key */
	andb	%bh, %al
	cmpb	%al, %bl
	pushf			/* Save ZF */
	je	4f
	/* Not the specified key: remove from buffer and stop waiting */
	movb	$0x10, %ah
	int	$0x16
	jmp	98f
4:	/* Wait until the Ctrl key is no longer depressed */
	movw	$4, %cx
5:	decw	%cx
	js	98f
	/* Is the Ctrl key pressed? */
	movb	$0x12, %ah
	int	$0x16
	testb	$0x4, %al
	je	98f
	call	wait_for_tick
	jnz	5b
98:	popf			/* Restore ZF */
99:	/* Restore registers and return */
	popw	%ax
	ret
	.size wait_for_key, . - wait_for_key

/* Wait for timer tick
 *
 * Used by wait_for_key
 */
wait_for_tick:
	pushl	%eax
	pushw	%fs
	movw	$0x40, %ax
	movw	%ax, %fs
	movl	%fs:(0x6c), %eax
1:	pushf
	sti
	hlt
	popf
	cmpl	%fs:(0x6c), %eax
	je	1b
	popw	%fs
	popl	%eax
	ret
	.size wait_for_tick, . - wait_for_tick
