/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * Driver for Solarflare network controllers and boards
 * Copyright 2019 Solarflare Communications Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <console.h>
#include <config/general.h>
#include <gpxe/init.h>
#include <gpxe/gdbstub.h>

/** @file
 *
 * GDB stub startup for sfconfig with no shell
 *
 */

void sf_gdbstub ( void ) {
	int k __unused;
	const char *trans_name = NULL;
#ifdef GDBUDP
	char netn[5];
#endif
	int argc = 0;
	char *argv[] = { NULL };
	struct gdb_transport *trans;

	printf ( "GDB stub:\n" );
#ifdef GDBSERIAL
	printf ( "- 'S' for serial port\n" );
#endif
#ifdef GDBUDP
	printf ( "- '0' for UDP over net0 (etc.)\n" );
#endif

	k = getkey ( 0 );

#ifdef GDBSERIAL
	if ( k == 'S' || k == 's' ) {
		trans_name = "serial";
		argc = 0;
	}
#endif
#ifdef GDBUDP
	if ( k >= '0' && k <= '9' ) {
		trans_name = "udp";
		sprintf ( netn, "net%c", k );
		argc = 1;
		argv[0] = netn;
	}
#endif

	if ( !trans_name )
		return;

	/* Initialise transport */
	trans = find_gdb_transport ( trans_name );
	if ( !trans ) {
		printf ( "%s: no such transport (is it compiled in?)\n",
			 trans_name );
		goto fail;
	}

	printf ( "Starting GDB %s transport\n", trans_name );

	if ( trans->init ) {
		if ( trans->init ( argc, argv ) != 0 ) {
			goto fail;
		}
	}

	/* Enter GDB stub */
	gdbstub_start ( trans );
	return;

fail:
	/* Pause to see errors */
	mdelay ( 5000 );
	return;
}

