/*
 * SPDX-License-Identifier: GPL-2.0+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * You can also choose to distribute this program under the terms of
 * the Unmodified Binary Distribution Licence (as given in the file
 * COPYING.UBDL), provided that you have satisfied its requirements.
 */
#ifndef _GPXE_PARAMS_H
#define _GPXE_PARAMS_H

/** @file
 *
 * Form parameters
 *
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <gpxe/list.h>
#include <gpxe/refcnt.h>

/** A form parameter list */
struct parameters {
	/** List of all parameter lists */
	struct list_head list;
	/** Name */
	const char *name;
	/** Parameters */
	struct list_head entries;
};

/** A form parameter */
struct parameter {
	/** List of form parameters */
	struct list_head list;
	/** Key */
	const char *key;
	/** Value */
	const char *value;
};

/** Iterate over all form parameters in a list */
#define for_each_param( param, params )				\
	list_for_each_entry ( (param), &(params)->entries, list )

extern struct parameters * find_parameters ( const char *name );
extern struct parameters * create_parameters ( const char *name );
extern struct parameter * add_parameter ( struct parameters *params,
					  const char *key, const char *value );
extern void destroy_parameters ( struct parameters *params );
extern void claim_parameters ( struct parameters *params );

#endif /* _GPXE_PARAMS_H */
